# $Id: GSMI.pm,v 1.2 2005/05/16 13:31:10 elwood Exp $
# GSM-Interface this is what is used to connect to every kind of phone

package GSMI;

#use strict;
use Carp;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);

require Exporter;
require DynaLoader;
require AutoLoader;

package GSMI;
@ISA = qw(Exporter DynaLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	
);
$VERSION = '0.1';

sub AUTOLOAD {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.  If a constant is not found then control is passed
    # to the AUTOLOAD in AutoLoader.

    my $constname;
    my $self = shift(@_); # nicht shift cause AUTOLOAD is different
    ($constname = $AUTOLOAD) =~ s/.*:://;
    print "Auto was called with $constname\n";

    
#    my $val = constant($constname, @_ ? $_[0] : 0);
#    if ($! != 0) {
#	if ($! =~ /Invalid/) {
#	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
#	    goto &AutoLoader::AUTOLOAD;
#	}
#	else {
#		croak "Your vendor has not defined Gnokii::GSM macro $constname";
#	}
#    }
#    eval "sub $self->{'connection'}->$AUTOLOAD { $val }";
    $result = $self->{'connection'}->$constname(@_);
    return($result);
    goto &$AUTOLOAD;
}

sub DESTROY
{
    my $self = shift;
    $self->{'connection'}->disconnect();
}

# Connect is called like this: my $conn = GSMI->connect("driver=Gnokii;device=/dev/ttyS0;options=model:6310,connection:IRDA")
sub connect
{
    my $class = shift; 
    my $self = {};
    my $driveroptions = shift;

    my @options = split(/;/, $driveroptions);
    foreach my $o (@options)
    {
	my @parts = split(/=/, $o);
	if($parts[0] eq 'driver')
	{
	    my $driverworks = eval("require GSMD::$parts[1]");
	    if($driverworks)
	    {
		$self->{'driverprefix'} = "GSMD::$parts[1]";
	    }
	    else
	    {
		croak "Could not load driver GSMD::$parts[1]";
	    }
	}
	elsif($parts[0] eq 'device')
	{
	    # We need to be able to read and write on this device
	    if(((-r $parts[1]) && (-w $parts[1])) || (! $parts[1] =~ /^\/dev/))
	    {
		$self->{'device'} = $parts[1];
	    }
	    else
	    {
		croak "Can not access device please check permissions";
	    }
	}
	elsif($parts[0] eq 'options')
	{
	    my %options;

	    foreach my $o (split(/,/, $parts[1]))
	    {
		my @parts = split(/:/, $o);
		$options{$parts[0]} = $parts[1];
	    }
	    $self->{'driveroptions'} = \%options;
	}
	else
	{
	    croak "Unknown conn-parameter usage: connect->{\"driver=drivername;devicedevicename;options=key1:value1,key2:value2...\")";
	}
    }
    if(!defined($self->{'driverprefix'}) || !defined($self->{'device'}))
    {
	croak "Parameter missing";
    }
    else # we have everything we need, so we can call connect in the Driver
    {
	$self->{'connection'} = $self->{'driverprefix'}->connect($self->{'device'}, $self->{'driveroptions'});
    }
    if($self->{'connection'})
    {
	bless $self, $class;
	return($self);
    }
    else
    {
	return(undef);
    }
}

1;
__END__
 
=head1 NAME

GSMI - Perl-Interface for accessing Your mobile phone

=head1 SYNOPSIS

  use GSMI;

=head1 DESCRIPTION

GSMI is the Interface (comparable to DBI) for accessing Your mobile
phone's address book, SMS-capability etc. It loads the driver specified
in the initialization to do the actual work. The method calls are then 
dispatched to the underlying GSMD-Module.

=head1 CONSTRUCTOR

  $gsm = GSMI->connect("driver=DRIVERNAME;device=DEVICENAME;options=driveroption1:drivervalue1,driveroption2:drivervalue2...);

DRIVERNAME is the name of the the driver in the GSMD-hierarchy (eg Gnokii).
DEVICENAME is the name of the device used to access the phone eg /dev/ttyS0.
The possible options are explained in the manpage of the GSMD-module. They are
key-value pairs which are separated by a colon. The pairs are separated by semi-
colons.

=head1 METHODS

  $gsm->GetIMEI($emptyhash);

This method returns a hash containing the IMEI, model, revision and
manufacturer of the phone. The returned hash looks as follows:

  $emptyhash->{'imei'}
  $emptyhash->{'model'}
  $emptyhash->{'revision'}
  $emptyhash->{'manufacturer'}

  $gsm->GetSMSCenter($emptyarray, $start, $ende);

This method fills $emptyarray with the messagecenters in hashes stored in the phone.
The hashes have the following entries:

  $emptyhash->{'name'}
  $emptyhash->{'no'}
  $emptyhash->{'defaultname'}
  $emptyhash->{'format'}
  $emptyhash->{'validity'}
  $emptyhash->{'type'}
  $emptyhash->{'smscnumber'}
  $emptyhash->{'recipienttype'}
  $emptyhash->{'recipientnumber'}

  $gsm->GetAlarm($emptyhash)

This method fills a hash with the date of the alarm in epoch-format.
The hash has the following entries:

  $emptyhash->{'date'}

  $gsm->GetRF($emptyhash)

This method fills the hash with values for the current level of radio
reception. The hash has the following entries:

  $emptyhash->{'level'}

  $gsm->GetPowerStats($emptyhash)

This method fills the hash with values for the batterylevel of the  phone.
The hash's entries are:

  $emptyhash->{'level'}
  $emptyhash->{'source'}

  $gsm->GetSMSStatus($emptyhash)

This method fills the hash with values for the number of read/unread SMSs.
The hash has the following entries:

  $emptyhash->{'read'}
  $emptyhash->{'unread'}

  $gsm->GetNetworkInfo($emptyhash)

This method fills the hash with information about the network, that the phone
is booked into. The hash has the following entries:

  $emptyhash->{'name'}
  $emptyhash->{'countryname'}
  $emptyhash->{'networkcode'}
  $emptyhash->{'cellid'}
  $emptyhash->{'lac'}

  $gsm->WritePhonenookEntry($phonebookhash)

This method writes a new entry to the phonebook. $phonebookhash has to be filled
with the desired values of the entry. 'location' is the number of the phonebook entry.
Be aware that this will overwrite an existing entry in the phone. Memorytype can be SM for
simcard and ME for Phone memory. $phonebookhash has the following entries:

  $phonebookhash->{'location'} 
  $phonebookhash->{'number'} 
  $phonebookhash->{'callergroup'} 
  $phonebookhash->{'name'} 
  $phonebookhash->{'memorytype'} 

  $gsm->WriteCalendarNote($location, $calendarhash)

This method write a new entry to the calendar. $calendarhash has to be filled with
the values for this entry. $location is the number of the entry. Existing entries will
be overwritten. 'date' is written in epoch-format (seconds since 1/1/1970). 'recurrence' 
can be one of: NEVER, DAILY, WEEKLY, 2WEEKLY, YEARLY. 'type' can be any of: CALL, MEETING or
BIRTHDAY. 'number' only has a meaning in if 'type' is CALL. The hash has the following entries:

  $calendarhash->{'text'}
  $calendarhash->{'date'}
  $calendarhash->{'type'}
  $calendarhash->{'number'}
  $calendarhash->{'recurrence'}

  $gsm->GetCalendarNote($start, $ende, $emptyarray)

This methods fills the array with hash entries. The entries numbered from $start to $end are
read from the phone. The hash entries of the array will have the structure shown in WriteCalendarNote.

  $gsm->DeleteAllTodos()

This method removes all Todo-Entries on the phone.

  $gsm->WriteTodo($todohash)

This method adds a Todo to the list on the phone. The hash passed to it
contains the details of the todo and has the following format:

  $todohash->{'text'}
  $todohash->{'priority'}

  $gsm->GetTodo($start, $end, $emptyarray)

This method gets the todos starting from index $start up to index $end. The
results are placed in the array which contains hashes. The hashes are of the same
form as shown in WriteTodo.

'priority' can be one of HIGH, MEDIUM, LOW.

  $gsm->SetAlarm($time)

This method sets the alarm on the phone to $time. $time is in the epoch format.

  $gsm->CreateSMSFolder($name)

On phones that support this, a new SMS-Folder with name $name is created.

  $gsm->GetSpeedDial($index, $emptyhash)

This method fills $emptyhash with the data of the SpeedDial setting at $index. The
hash has the following entries:

  $emptyhash->{'number'}
  $emptyhash->{'location'}
  $emptyhash->{'memory'}

Note that this is just a reference to a phonebook entry and number does not mean the
actual phonenumber.

  $gsm->SetSpeedDial($number, $location, $memory)

This method sets the speed-dial $number (the key on Your phone) to the entry $location in 
memorytype $memory. $memory can be one of SM or ME.

  $gsm->WriteWapSetting($index, $waphash)

On phones that support this a new wapsetting is created on the phone at $location. An existing
entry at this memory location will be overwritten. $wapentry has the following format;

  $waphash->{'gsm_data_authentication'}
  $waphash->{'password'}
  $waphash->{'smsservicenumber'}
  $waphash->{'number'}
  $waphash->{'name'}
  $waphash->{'ip'}
  $waphash->{'gprs_connection'}
  $waphash->{'smsservernumber'}
  $waphash->{'session'}
  $waphash->{'bearer'}
  $waphash->{'gsm_data_username'}
  $waphash->{'location'}
  $waphash->{'call_speed'}
  $waphash->{'gprs_login'}
  $waphash->{'gsm_data_login'}
  $waphash->{'call_type'}
  $waphash->{'home'}
  $waphash->{'username'}
  $waphash->{'apn'}
  $waphash->{'gsm_data_password'}
  $waphash->{'gprs_authentication'}
  $waphash->{'gsm_data_ip'}


'session' is one of TEMPORARY,PERMANET. 'security' is a boolean value. 'bearer' is one of 
GSM DATA, GPRS, SMS. 'gsm_data_authentication' and 'gprs_authentication' are a boolean values.
'call_type' is either ISDN or an analogue call will be made for all other types. 'gsm_data_login' and
 'gprs_login' are either MANUAL or an automatic login takes place. 'gprs_connection' is either 'allways'
or is set to 'when needed'.

  $gsm->ActivateWapSetting($index)

This method activates the WAP-Setting number $index.

  $gsm->GetWapSetting($index, $emptyhash)

This method reads the WAP setting at position $index. The $emptyhash has the same structure as in WriteWapSetting.

  $gsm->DeleteWapBookmark($index)

This method deletes the bookmark at location $index.

  $gsm->WriteWapBookmark($waphash)

This method adds the contents of $waphash to the WAP bookmarks. When the method was successfull, 
the hash contains an additional field 'number' that gives the index of the added bookmark. $waphash
has the following entries:

  $waphash->{'name'}
  $waphash->{'url'}

  $gsm->GetWapBookMark($index, $emptyhash)

This method reads the bookmark at $index and puts the contents in $emptyhash which has the format described
at WriteWapBookmark.

  $gsm->SetDateTime($time)

This method sets the date and time of the phone to the value of $time. $time is in the epoch-format.

  $gsm->GetDateTime($time)

This method reads date and time from the phone and stores them in $time in epoch-format.

  $gsm->GetProfile($start, $end, $emptyarray)

This method loads the userprofiles in the phone starting from index $start up to index $end.
The array contains hashes. The hashes have the following format:

  $hash->{'number'}
  $hash->{'name'}
  $hash->{'defaultname'}
  $hash->{'callalert'}
  $hash->{'ringtonenumber'}
  $hash->{'volume_level'}
  $hash->{'message_tone'}
  $hash->{'keypad_tone'}
  $hash->{'warning_tone'}
  $hash->{'vibration'}
  $hash->{'automatic_answer'}
  $hash->{'caller_group'}

  $gsm->SendSMS($smshash)

This method is used for sending a SMS. The data for the SMS is stored in $smshash which has the following
format:

  $smshash->{'destination'}  # phonenumber
  $smshash->{'smscnumber'}
  $smshash->{'smscindex'}    # use this instead of smscnumber to user the smsc at this index 
  $smshash->{'message'}
  $smshash->{'animation'}    # Use this instead of message to send an animation
  $smshash->{'report'}       # To request a delivery report
  $smshash->{'class'} 
  $smshash->{'validity'}     # validity in seconds
  $smshash->{'eighbit'}      # boolean

  $gsm->GetSMS($memorytype, $index, $emptyhash)

This method reads an SMS from the memory $memorytype at position $index into $emptyhash.
$emptyhash has the same entries as in SendSMS. $memorytype can be on of the following:
SM, ME, IN, OU, AR, TE and F1-FX for userdefined folders if the phone supports this.

  $gsm->ReadPhoneBook($memory, $start, $end, $emptyarray)

This method reads the phonebook entries of the phone on memory type $memory. $memory can be one
of ME, SM, FD, ON, EN, DC, RC, MC, LD. It depends on the model of the phone and the implementation
of the driver which will work. The results are put in an array, which has hashes as entries. On hash 
consists of the following entries:

  $hash->{'name'}
  $hash->{'number'}
  $hash->{'memorytype'}
  $hash->{'location'}
  $hash->{'group'}

  $gsm->GetLogo($logodata, $emptyhash)

This method reads the logo described by the hash $logodata into $emptyhash. $logodata has one
entry: $logodata->{'type'} which can be one of text, dealer, op, startup, caller, picture, emspicture,
emsanimation, If type is caller an index must be supplied at $logodata->{'callerindex'). The returned hash,
contains the bitmap in $emptyhash->{'text'} if the logo was of type text.
If the logo was of type networkcode and networkname are stored in $emptyhash->{'netcode'} and $emptyhash->{'netname'}.
The hash allways has the  following entries:

  $emptyhash->{'type'}
  $emptyhash->{'bitmap'}
  $emptyhash->{'size'}
  $emptyhash->{'height'}
  $emptyhash->{'width'}

  $gsm->GetRingTone($index, $emptyhash)

This method reads rington number $index into $emptyhash. The hash has the
following entries:
  
  $emptyhash->{'name'}
  $emptyhash->{'ringtone'}

  $gsm->disconnect().

This routine terminates the session with the phone it should allways be called at the end of a program.

  $emptyhash->{'simused'}
  $emptyhash->{'simfree'}
  $emptyhash->{'phoneused'}
  $emptyhash->{'
